"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
/* class decorator */
function staticImplements() {
    return function (constructor) { };
}
var LazyPromise = /** @class */ (function () {
    /**
     * Creates a new LazyPromise with the standard Promise constructor, except that executor
     * is deferred until .then(), .catch(), or .finally() is called.
     * @param executor {function} Callback used to initialize the promise. This callback is passed two arguments:
     * a resolve callback used resolve the promise with a value or the result of another promise,
     * and a reject callback used to reject the promise with a provided reason or error.
     */
    function LazyPromise(executor) {
        this.promise = null;
        this.promiseCreator = function () { return new Promise(executor); };
    }
    LazyPromise_1 = LazyPromise;
    /**
     * Creates a new LazyPromise from a function that returns a Promise.
     * @param promiseCreator {function} Function that the returns the Promise to be deferred until .then(), .catch(),
     * or .finally() is called.
     */
    LazyPromise.fromPromiseCreator = function (promiseCreator) {
        return new LazyPromise_1(function (resolve, reject) {
            promiseCreator().then(resolve, reject);
        });
    };
    /**
     * Calls the executor if it has not been executed.
     * Attaches callbacks for the resolution and/or rejection of the resulting Promise.
     * @param onFulfilled {function} The callback to execute when the Promise is resolved.
     * @param onRejected {function} The callback to execute when the Promise is rejected.
     * @returns {Promise} A Promise for the completion of which ever callback is executed.
     */
    LazyPromise.prototype.then = function (onFulfilled, onRejected) {
        if (!this.promise) {
            this.promise = this.promiseCreator();
        }
        return this.promise.then(onFulfilled, onRejected);
    };
    /**
     * Calls the executor if it has not been executed.
     * Attaches a callback for only the rejection of the resulting Promise.
     * @param onRejected {function} The callback to execute when the Promise is rejected.
     * @returns {Promise} A Promise for the completion of the callback.
     */
    LazyPromise.prototype.catch = function (onRejected) { return this.then().catch(onRejected); };
    ;
    /**
     * Calls the executor if it has not been executed.
     * Attaches a callback that is invoked when the resulting Promise is settled (fulfilled or rejected). The
     * resolved value cannot be modified from the callback.
     * @param onFinally {function} The callback to execute when the Promise is settled (fulfilled or rejected).
     * @returns {Promise} A Promise for the completion of the callback.
     */
    LazyPromise.prototype.finally = function (onFinally) { return this.then().finally(onFinally); };
    /**
     * Creates a new resolved LazyPromise for the provided value.
     * Equivalent to Promise.resolve() because value is already executed
     * @param value {any} The value which is resolved
     * @returns {Promise} A promise whose internal state matches the provided value.
     */
    LazyPromise.resolve = function (value) {
        return LazyPromise_1.fromPromiseCreator(function () { return Promise.resolve(value); });
    };
    /**
     * Creates a new rejected LazyPromise for the provided value.
     * Equivalent to Promise.reject() because value is already executed
     * @param reason {any} The value which is rejected
     * @returns {LazyPromise} A promise whose internal state matches the provided value.
     */
    LazyPromise.reject = function (reason) {
        return LazyPromise_1.fromPromiseCreator(function () { return Promise.reject(reason); });
    };
    LazyPromise.all = function (values) {
        return LazyPromise_1.fromPromiseCreator(function () { return Promise.all(values); });
    };
    LazyPromise.race = function (values) {
        return LazyPromise_1.fromPromiseCreator(function () { return Promise.race(values); });
    };
    var LazyPromise_1;
    LazyPromise = LazyPromise_1 = __decorate([
        staticImplements()
    ], LazyPromise);
    return LazyPromise;
}());
exports.LazyPromise = LazyPromise;
Object.setPrototypeOf(LazyPromise.prototype, Promise.prototype);
Object.setPrototypeOf(LazyPromise, Promise);
/**
 * Alias for LazyPromise.fromPromiseCreator that creates a new LazyPromise from a function that returns a Promise.
 * @param promiseCreator {function} Function that the returns the Promise to be deferred until .then(), .catch(), or .finally() is called.
 * @returns {LazyPromise}
 */
function lz(promiseCreator) {
    return LazyPromise.fromPromiseCreator(promiseCreator);
}
exports.lz = lz;
//# sourceMappingURL=LazyPromise.js.map